package cn.bunny.security.config;

import cn.bunny.security.custom.CustomPasswordEncoder;
import cn.bunny.security.handelr.*;
import cn.bunny.security.service.UserDetailsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.core.session.SessionRegistry;
import org.springframework.security.core.session.SessionRegistryImpl;
import org.springframework.security.web.SecurityFilterChain;

import static org.springframework.security.config.Customizer.withDefaults;

@Configuration
@EnableWebSecurity
@EnableMethodSecurity
public class WebSecurityConfig {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private CustomPasswordEncoder customPasswordEncoder;
    @Autowired
    private AuthenticationConfiguration authenticationConfiguration;

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity httpSecurity) throws Exception {
        httpSecurity.authorizeHttpRequests(authorize -> {
            authorize.requestMatchers("/", "/test/**", "/diagram-viewer/**", "/editor-app/**", "/*.html",
                    "/admin/system/index/login",
                    "/favicon.ico", "/swagger-resources/**", "/webjars/**", "/v3/**", "/swagger-ui.html/**", "/doc.html").permitAll().anyRequest().authenticated();
        });

        // 注销登录
        httpSecurity
                .logout(logout -> {
                    logout.logoutSuccessHandler(new SecurityLogoutSuccessHandler());
                })
                .exceptionHandling(exception -> {
                    // 请求未授权接口
                    exception.authenticationEntryPoint(new SecurityAuthenticationEntryPoint());
                    // 没有权限访问
                    exception.accessDeniedHandler(new SecurityAccessDeniedHandler());
                })
                // 后登录的账号会使先登录的账号失效
                .sessionManagement(session -> {
                    // 禁用session
                    // session.sessionCreationPolicy(SessionCreationPolicy.STATELESS);
                    // 最大登录数为1
                    session.maximumSessions(1)
                            // 可以获取到所有登录的用户，以及登录状态，设置session状态
                            .sessionRegistry(sessionRegistry())
                            // 有相同用户已登录时
                            .expiredSessionStrategy(new SecuritySessionInformationExpiredStrategy());
                    // 会话失效，同时内容
                    session.invalidSessionStrategy(new SecurityInvalidSessionStrategy());
                });
        // 关闭csrf攻击
        httpSecurity.csrf(AbstractHttpConfigurer::disable);
        // 跨域访问权限
        httpSecurity.cors(withDefaults());
        // 自定义用户认证和密码
        httpSecurity.userDetailsService(userDetailsService).passwordManagement(customPasswordEncoder);
        // 记住我
        httpSecurity.rememberMe(e -> e.rememberMeParameter("rememberBunny").rememberMeCookieName("rememberBunny").key("BunnyKey"));
        // 自定义过滤器
        // httpSecurity.addFilterAt(loginFilter(), UsernamePasswordAuthenticationFilter.class);
        // httpSecurity.addFilterBefore(new TokenAuthenticationFilter(redisTemplate), UsernamePasswordAuthenticationFilter.class);
        // httpSecurity.addFilter(new TokenLoginFilter(authenticationConfiguration, redisTemplate));

        return httpSecurity.build();
    }

    @Bean
    public SessionRegistry sessionRegistry() {
        return new SessionRegistryImpl();
    }
}
